; Solution:
; - still display three  copies of P1 (color the first ones like the PF)
; - don't move P1, when moving the players left for the first time

    processor 6502
    include vcs.h


SMOOTH  = 1     ; set 1 for smooth diagonals

BLACK   = $00
WHITE   = $0c
GREEN   = $c2
RED     = $42
YELLOW  = $1c
BLUE    = $80
ORANGE  = $36
; two extra colors to prove the concept:
MAGENTA = $66
CYAN    = $aa


;===============================================================================
; Z P - V A R I A B L E S
;===============================================================================

    SEG.U   variables
    ORG     $80

tmpVar      .byte
tmpVar2     .byte
frameCnt    .byte
random      .byte

faceLst     ds 9*3
faceTLst    = faceLst
faceLLst    = faceLst+9
faceRLst    = faceLst+18


  MAC SLEEP
    IF {1} = 1
      ECHO "ERROR: SLEEP 1 not allowed !"
      END
    ENDIF
    IF {1} & 1
      nop $00
      REPEAT ({1}-3)/2
        nop
      REPEND
    ELSE
      REPEAT ({1})/2
        nop
      REPEND
    ENDIF
  ENDM


;===============================================================================
; R O M - C O D E
;===============================================================================
    SEG     Bank0
    ORG     $f000

Start SUBROUTINE
    sei                         ;           Disable interrupts, if there are any.
    cld                         ;           Clear BCD math bit.
    ldx     #0
    txs
    pha                         ;           Set stack to beginning.
    txa
.clearLoop:
    pha
    dex
    bne     .clearLoop
    jsr     GameInit
.mainLoop:
    jsr     VerticalBlank
    jsr     GameCalc
    jsr     DrawScreen
    jsr     OverScan
    jmp     .mainLoop


GameInit SUBROUTINE
    lda     #1
    sta     random
    sta     frameCnt
    rts


VerticalBlank SUBROUTINE
    lda     #2
    sta     WSYNC
    sta     VSYNC
    sta     WSYNC
    sta     WSYNC
    lsr
    ldx     #43
    sta     WSYNC
    sta     VSYNC
    stx     TIM64T
    rts


NextRandom SUBROUTINE
    lda    random           ; 3             generate next random number
    asl                     ; 2
    asl                     ; 2
    asl                     ; 2
    eor    random           ; 3
    asl                     ; 2
    rol    random           ; 5
    rts


GameCalc SUBROUTINE
    jsr     NextRandom
    dec     frameCnt

    bit     SWCHB
    bpl     .static
    lda     frameCnt
    and     #$1f
    bne     .skipColor

    ldx     #9*3-1
.loopColors:
    jsr     NextRandom
    lda     random
    and     #$07
    bit     SWCHB
    bvc     .allColors
    cmp     #6
    bcs     .loopColors
.allColors:
    tay
    lda     ColorTbl,y
    sta     faceLst,x
    dex
    bpl     .loopColors
.skipColor:
    rts

.static:
    ldx     #9*3-1
.loopStatic:
    lda     FaceColorTbl,x
    sta     faceLst,x
    dex
    bpl     .loopStatic
    rts


FaceColorTbl:
    .byte #YELLOW,#MAGENTA,#GREEN,#RED,#ORANGE,#BLUE,#MAGENTA,#GREEN,#WHITE
    .byte #BLUE,#WHITE,#GREEN,#RED,#ORANGE,#CYAN,#YELLOW,#GREEN,#BLUE
    .byte #ORANGE,#YELLOW,#CYAN,#GREEN,#BLUE,#MAGENTA,#WHITE,#MAGENTA,#RED


    align 256

;    ds  58, 0

DrawScreen SUBROUTINE
    tsx
    stx     tmpVar          ;       save stack pointer

    ldx     #228
.waitTim:
    lda     INTIM
    bne     .waitTim
    sta     WSYNC
    sta     VBLANK
    stx     TIM64T

;-------------------------------------
    ldy     #4-1
.loop1a:
    sta     WSYNC
;---------------------------------------
    sta     HMOVE
    lda     PxPat2,y
    sta     GRP0
    sta     GRP1
    ldx     #0
    stx     COLUP1
    lda     #%10
    sta     ENABL
    SLEEP   10
    lda     faceTLst+2      ; 3     GREEN
    sta     COLUP0
    sta     COLUP1
    SLEEP   5
    stx     COLUP0
    dey
    bpl     .loop1a

    ldy     #3-1
.loop1b:
    sta     WSYNC
;---------------------------------------
    sta     HMOVE           ; 3
    lda     #%10
    sta     ENAM0
    sta     ENAM1
    stx     COLUP1
    SLEEP   21
    lda     faceTLst+2      ; 3     GREEN
    sta     COLUP0
    sta     COLUP1
    SLEEP   3
    stx     COLUP0
    dey
    bpl     .loop1b

    sta     WSYNC
;---------------------------------------
    sta     HMOVE
    stx     GRP0
    stx     GRP1
    stx     ENAM0
    stx     ENAM1
    lda     faceTLst+2      ; 3     GREEN
    sta     COLUPF
    lda     #$f0
    sta     PF2
    lda     #$40
    sta     HMM0
    lda     #$c0
    sta     HMM1

;***************************************************************
    ldy     #4-1
.loop1c:
    sta     WSYNC
;---------------------------------------
    sta     HMOVE
    lda     PxPat2,y
    sta     GRP0
    sta     GRP1
    stx     COLUP1
    SLEEP   17
    lda     faceTLst+1      ; 3     MAGENTA
    sta     COLUP0
    sta     COLUP1
    lda     faceTLst+5      ; 3     BLUE
    sta     COLUP0
    sta     COLUP1
    stx.w   COLUP0
    dey
    bpl     .loop1c

    stx     HMM0
    stx     HMM1

    ldy     #3-1
.loop1d:
    sta     WSYNC
;---------------------------------------
    sta     HMOVE           ; 3
    lda     #%10
    sta     ENAM0
    sta     ENAM1
    stx     COLUP1
    SLEEP   18
    lda     faceTLst+1      ; 3     MAGENTA
    sta     COLUP0
    sta     COLUP1
    lda     faceTLst+5      ; 3     BLUE
    sta     COLUP0
    sta     COLUP1
    SLEEP   2
    stx     COLUP0
    dey
    bpl     .loop1d

    sta     WSYNC
;---------------------------------------
    sta     HMOVE
    stx     GRP0
    stx     GRP1
    stx     ENAM0
    stx     ENAM1
    lda     #$7f
    sta     PF2
    lda     faceTLst+1      ; 3     MAGENTA
    sta     COLUPF
    lda     #$40
    sta     HMM0
    lda     #$c0
    sta     HMM1
    SLEEP   5
    lda     faceTLst+5      ; 3     BLUE
    sta     COLUBK
    sta     COLUPF
    lda     #$ff
    stx     COLUBK
    sta     PF2

;***************************************************************
    ldy     #4
.loop2a:
    sta     WSYNC
;---------------------------------------
    sta     HMOVE
    lda     PxPat2-1,y
    sta     GRP0
    sta     GRP1
    lda     faceTLst+1      ; 3     MAGENTA
    sta     COLUPF
    lda     faceTLst+0      ; 3     YELLOW
    sta     COLUP0
    sta     COLUP1
    ldx     faceTLst+8      ; 3     WHITE
    txs                     ; 2     used later!
    SLEEP   2               ;       -3..+4
    lda     faceTLst+4      ; 3     ORANGE
    sta     COLUP0
    sta     COLUP1
    lda     faceTLst+5      ; 3     BLUE
    sta     COLUPF
    stx     COLUP0
    stx     COLUP1
    dey
    bne     .loop2a

    sty     HMM0
    sty     HMM1

    ldy     #5-1
.loop2b:
    sta     WSYNC
;---------------------------------------
    sta     HMOVE           ; 3
    lda     #%10
    sta     ENAM0
    sta     ENAM1
    lda     faceTLst+1      ; 3     MAGENTA
    sta     COLUPF
    SLEEP   9
    lda     faceTLst+0      ; 3     YELLOW
    sta     COLUP0
    sta     COLUP1
    lda     faceTLst+4      ; 3     ORANGE
    sta     COLUP0
    sta     COLUP1
    lda     faceTLst+5      ; 3     BLUE
    sta     COLUPF
    stx     COLUP0
    stx     COLUP1
    dey
    bne     .loop2b

;***************************************************************

    sta     HMCLR
    sty     ENAM0
    sty     ENAM1

    sta     WSYNC
;---------------------------------------
;    sta     HMOVE           ; 3
    lda     #$0f
    sta     PF1

    lda     #%11000000
    sta     GRP0
    sta     GRP1
    lda     faceLLst+0      ; 3     BLUE
    sta     COLUP0
    lda     faceTLst+0      ; 3     YELLOW
    sta     COLUPF
    lda     faceTLst+3      ; 3     RED
    sta     COLUP1
    ldx     faceTLst+7      ; 3     GREEN
    sta     COLUP0
    lda     faceTLst+4      ; 3     ORANGE
    sta.w   COLUPF
    stx     COLUP1
    stx     COLUP0
    tsx                     ; 2     faceTLst+8
    stx     COLUPF
    lda     faceRLst+2      ; 3     CYAN
    sta     COLUP1

    lda     #$40
    sta     HMM0
    lda     #$c0
    sta     HMM1

    ldy     #2-1
.loop2c:
    sta     WSYNC
;---------------------------------------
    sta     HMOVE           ; 3
    lda     PxPat2+1,y
    sta     GRP0
    sta     GRP1
    lda     faceLLst+0      ; 3     BLUE
    sta     COLUP0
    lda     faceTLst+0      ; 3     YELLOW
    sta     COLUPF
    lda     faceTLst+3      ; 3     RED
    sta     COLUP1
    ldx     faceTLst+7      ; 3     GREEN
    sta     COLUP0
    lda     faceTLst+4      ; 3     ORANGE
    sta     COLUPF          ; 3/4
    stx     COLUP1
    stx     COLUP0
    tsx                     ; 2     faceTLst+8
    stx     COLUPF
    lda     faceRLst+2      ; 3     CYAN
    sta     COLUP1
    dey
    bpl     .loop2c

    ldy     #3-1
.loop2d:
    sta     HMCLR
    lda     #$e0            ;       move 2 pixels left
    sta     HMP0
    lda     #$20            ;       move 2 pixels right
    sta     HMP1
;    sta     WSYNC
;---------------------------------------
    sta     HMOVE           ; 3
    lda     #%10
    sta     ENAM0
    sta     ENAM1
    lda     faceLLst+0      ; 3     BLUE
    sta     COLUP0
    lda     faceTLst+0      ; 3     YELLOW
    sta     COLUPF
    lda     faceTLst+3      ; 3     RED
    sta     COLUP1
    ldx     faceTLst+7      ; 3     GREEN
    sta     COLUP0
    lda     faceTLst+4      ; 3     ORANGE
    sta     COLUPF
    stx     COLUP1
    stx     COLUP0
    tsx                     ; 2     faceTLst+8
    stx     COLUPF
    lda     faceRLst+2      ; 3     CYAN
    sta     COLUP1
    dey
    bpl     .loop2d

    sta     WSYNC
;---------------------------------------
;    sta     HMOVE           ; 3
    SLEEP   3
    lda     #%11111111
    sta     GRP0
    sta     GRP1
    ldy     faceRLst+2      ; 3     CYAN
    lda     faceLLst+0      ; 3     BLUE
    sta     COLUP0
    lda     faceTLst+0      ; 3     YELLOW
    sta     COLUPF
    ldx     faceTLst+3      ; 3     RED
    stx     COLUP1
    lda     faceTLst+7      ; 3     GREEN
    stx     COLUP0
    ldx     faceTLst+4      ; 3     ORANGE
    stx     COLUPF
    sta     COLUP1
    sta     COLUP0
    tsx                     ; 2     faceTLst+8
    stx     COLUPF
    sty     COLUP1

    ldx     faceRLst+1      ; 3     YELLOW
    txs                     ; 2     used later

    lda     #%00
    sta     ENAM0
    sta     ENAM1

    sta     WSYNC
;---------------------------------------
    sta     HMOVE           ; 3
    lda     faceLLst+0      ; 3     BLUE
    sta     COLUP0
    sta     COLUPF
    lda     #$c0
    sta     HMM0
    lda     #$40
    sta     HMM1
    SLEEP   2
    ldx     faceRLst+2      ; 3     CYAN
    lda     faceTLst+3      ; 3     RED
    sta     COLUP1
    sta     COLUPF
    sta     COLUP0
    lda     faceTLst+7      ; 3     GREEN
    sta     COLUP1
    sta     COLUPF
    sta     COLUP0
    stx     COLUP1
    lda     #%110010
    stx     COLUPF

;***************************************************************

    sta     NUSIZ0
    SLEEP   2
    ldy     #4
.loop3a:
    dey
    sty     tmpVar2
    SLEEP   3
;---------------------------------------
    sta     HMOVE
    lda     PxPat3,y
    ldy     faceRLst+2      ; 3     CYAN
    sta     GRP0
    sta     GRP1
    lda     faceLLst+0      ; 3     BLUE
    sta     COLUP1
    sta     COLUPF
    lda     faceLLst+1      ; 3     WHITE
    sta     COLUP0
    ldx     faceTLst+6      ; 3     MAGENTA
    lda     faceTLst+3      ; 3     RED
    sta     COLUPF          ; 3     !!!
    stx     COLUP1
    stx     COLUP0
    ldx     faceTLst+7      ; 3     GREEN
    stx     COLUPF          ; 3     !!!
    tsx                     ; 2     YELLOW
    stx     COLUP1
    SLEEP   2
    sty     COLUPF
    ldy     tmpVar2
    bne     .loop3a

    sty     HMM0
    sty     HMM1

    ldy     #3
.loop3b:
    sta     WSYNC
;---------------------------------------
    sta     HMOVE           ; 3
    lda     #%10
    sta     ENAM0
    sta     ENAM1
    lda     faceLLst+0      ; 3     BLUE
    sta     COLUP1
    sta     COLUPF
    lda     faceLLst+1      ; 3     WHITE
    sta     COLUP0
    lda     faceTLst+6      ; 3     MAGENTA
    ldx     faceTLst+3      ; 3     RED
    SLEEP   2
    stx     COLUPF          ; 3     !!!
    sta     COLUP1
    tsx                     ; 2     YELLOW
    sta     COLUP0
    lda     faceTLst+7      ; 3     GREEN
    sta     COLUPF          ; 3     !!!
    stx     COLUP1
    lda     faceRLst+2      ; 3     CYAN
    sta     COLUPF
    dey
    bne     .loop3b

    sty     ENAM0
    sty     ENAM1

    sta     WSYNC
;---------------------------------------
    sta     HMOVE           ; 3
    lda     #$f0
    sta     PF2
    lda     faceLLst+0      ; 3     BLUE
    sta     COLUP1
    sta     COLUPF
    ldx     faceLLst+1      ; 3     WHITE
    stx     COLUP0
    lda     #%110000
    sta     NUSIZ0
    lda     #$c0
    stx.w   COLUBK          ; 3     !!!
    ldx     faceTLst+6      ; 3     MAGENTA
    stx     COLUP1
    stx     COLUPF          ; 3     !!!
    tsx                     ; 2     YELLOW
    stx     COLUP1
    stx     COLUBK
    ldx     faceRLst+2      ; 3     CYAN
    stx     COLUPF
    ldx     #0
    stx     COLUBK

    sta     HMM0
    lda     #$40
    sta     HMM1

;***************************************************************
    ldy     #3
.loop3c:
    sta     WSYNC
;---------------------------------------
    sta     HMOVE
    stx     COLUP1
    lda     PxPat3,y
    sta     GRP0
    sta     GRP1
    lda     faceLLst+0      ; 3     BLUE
    sta     COLUPF
    lda     faceLLst+2      ; 3     GREEN
    sta     COLUP0
    lda     faceLLst+1      ; 3     WHITE
    sta     COLUBK          ; 3     !!!
    sta     COLUP1
    ldx     faceTLst+6      ; 3     MAGENTA
    stx     COLUPF          ; 3     !!!
    lda     faceRLst+0      ; 3     ORANGE
    sta     COLUP1
    tsx                     ; 2     YELLOW
    stx     COLUBK
    lda     faceRLst+2      ; 3     CYAN
    sta     COLUPF
    ldx     #0
    stx     COLUBK
    dey
    bpl     .loop3c

    stx     HMM0
    stx     HMM1
;---------------------------------------
    sta     HMOVE
    stx     COLUP1
    lda     #%10
    sta     ENAM0
    sta     ENAM1
    lda     faceLLst+0      ; 3     BLUE
    sta     COLUPF
    ldx     faceLLst+2      ; 3     GREEN
    stx     COLUP0
    lda     faceLLst+1      ; 3     WHITE
    sta     COLUBK          ; 3     !!!
    sta     COLUP1
    ldx     faceTLst+6      ; 3     MAGENTA
    stx     COLUPF          ; 3     !!!
    lda     faceRLst+0      ; 3     ORANGE
    sta     COLUP1
    tsx                     ; 2     YELLOW
    stx     COLUBK
    lda     faceRLst+2      ; 3     CYAN
    sta     COLUPF
    ldx     #0
    stx     COLUBK

    ldy     #2-1
.loop3d:
    sta     WSYNC
;---------------------------------------
    sta     HMOVE
    stx     COLUP1
    lda     faceLLst+0      ; 3     BLUE
    sta     COLUPF
    ldx     faceLLst+2      ; 3     GREEN
    stx     COLUP0
    lda     faceLLst+1      ; 3     WHITE
    SLEEP   10
    sta     COLUBK          ; 3     !!!
    sta     COLUP1
    ldx     faceTLst+6      ; 3     MAGENTA
    stx     COLUPF          ; 3     !!!
    lda     faceRLst+0      ; 3     ORANGE
    sta     COLUP1
    tsx                     ; 2     YELLOW
    stx     COLUBK
    lda     faceRLst+2      ; 3     CYAN
    sta     COLUPF
    ldx     #0
    stx     COLUBK
    dey
    bpl     .loop3d

;***************************************************************
    stx     GRP0
    stx     GRP1
    stx     ENAM0
    stx     ENAM1
;---------------------------------------
    lda     #$70
    sta     PF2
    lda     faceLLst+0      ; 3     BLUE
    sta     COLUPF
    lda     #$10            ; 2
    sta.w   HMP0            ; 3
    lda     faceRLst+0
    ldx     faceLLst+2      ; 3     GREEN
    txs
    ldx     faceLLst+1      ; 3     WHITE
    stx     COLUBK
    tsx                     ; 2     GREEN
    stx     COLUPF
    sta     COLUBK
    ldx     faceRLst+1      ; 3     YELLOW
    sta     COLUPF
    stx     COLUBK
    lda     faceRLst+2      ; 3     CYAN
    sta     COLUPF
    ldx     #0
    stx     COLUBK
    lda     #$10            ; 2
    sta     HMP1            ; 3

    sta     WSYNC
;---------------------------------------
    sta     HMOVE           ; 3
    lda     faceLLst+0      ; 3     BLUE
    sta     COLUPF          ; 3
    sta     COLUP1
    lda     faceRLst+2      ; 3     CYAN
    sta     COLUP0
    SLEEP   4
    lda     faceRLst+1      ; 3     YELLOW
    ldx     faceLLst+1      ; 3     WHITE
    ldy     #$a0            ; 2
    sty     HMP0            ; 3
    stx     COLUBK
    tsx                     ; 2     GREEN
    stx     COLUPF
    ldx     faceRLst+0      ; 3     ORANGE
    stx     COLUBK
    stx     COLUPF
    sta     COLUBK
    lda     faceRLst+2      ; 3     CYAN
    sta     COLUPF
    ldx     #0
    stx     COLUBK
    lda     #$e0
    sta     HMP1

    ldy     #6
.loop4a:
    sta     WSYNC
;---------------------------------------
    sta     HMOVE
    lda     #%110101        ; 2     8 pixel wide missiles/double sized player
    sta     NUSIZ0
    sta     NUSIZ1
    lda     faceLLst+0      ; 3     BLUE
    sta     COLUPF
    SLEEP   10
    lda     faceRLst+0      ; 3     ORANGE
    ldx     faceLLst+1      ; 3     WHITE
    stx     COLUBK
    tsx                     ; 2     GREEN
    stx     COLUPF
    sta     COLUBK
    ldx     faceRLst+1      ; 3     YELLOW
    sta     COLUPF
    stx     COLUBK
    lda     faceRLst+2      ; 3     CYAN
    sta     COLUPF
    ldx     #0
    stx     COLUBK
    dey
    bne     .loop4a

    sta     WSYNC
;---------------------------------------
    sta     HMOVE
    lda     faceLLst+0
    sta     COLUPF
;    lda     #$e0
;    sta     HMP1
    SLEEP   5
    lda     #$20
    sta     HMP0

    lda     #%1000
    sta     REFP0
    stx     REFP1           ;       player 1 is NOT reflected

    lda     faceRLst+0
    ldx     faceLLst+1
    stx     COLUBK
    tsx                     ; 2     GREEN
    stx     COLUPF
    sta     COLUBK
    ldx     faceRLst+1      ; 3     YELLOW
    sta     COLUPF
    stx     COLUBK
    lda     faceRLst+2
    sta     COLUPF
    ldx     #0
    stx     COLUBK

    ldy     #6
.loop4b:
    sta     WSYNC
;---------------------------------------
    sta     HMOVE           ; 3
    lda     faceLLst+0+3    ; 3
    sta     COLUPF          ; 3
    lda     PxPat,y         ; 4
    sta     GRP0            ; 3
    sta     GRP1            ; 3
    SLEEP   8               ; 3
    lda     faceRLst+0      ; 3
    ldx     faceLLst+1      ; 3
    stx     COLUBK          ; 3     @
    tsx                     ; 2     GREEN
    stx     COLUPF
    sta     COLUBK
    ldx     faceRLst+1      ; 3     YELLOW
    sta     COLUPF
    stx     COLUBK
    lda     faceRLst+2+3
    sta     COLUPF
    ldx     #0
    stx     COLUBK
    dey
    bpl     .loop4b

    lda     faceLLst+0+3
    sta     COLUPF
    SLEEP   2
;---------------------------------------
    sta     HMOVE
    stx     GRP0
    stx     GRP1
    lda     faceLLst+1
    sta     COLUP1
    lda     faceRLst+1
    sta     COLUP0
    lda     faceRLst+0
    SLEEP   6
    ldx     faceLLst+1
    stx     COLUBK
    tsx                     ; 2     GREEN
    stx     COLUPF
    sta     COLUBK
    ldx     faceRLst+1
    sta     COLUPF
    stx     COLUBK
    lda     faceRLst+2+3
    sta     COLUPF
    ldx     #0
    stx     COLUBK

    ldy     #6
.loop4c:
    sta     WSYNC
;---------------------------------------
    sta     HMOVE
    lda     faceLLst+0+3
    sta     COLUPF
    lda     PxPat,y
    sta     GRP0
    sta     GRP1
    SLEEP   8
    lda     faceRLst+0
    ldx     faceLLst+1+3
    stx     COLUBK
    tsx                     ; 2     GREEN
    stx     COLUPF
    sta     COLUBK
    ldx     faceRLst+1+3
    sta     COLUPF
    stx     COLUBK
    lda     faceRLst+2+3
    sta     COLUPF
    ldx     #0
    stx     COLUBK
    dey
    bpl     .loop4c

    lda     faceLLst+0+3
    sta     COLUPF
    SLEEP   2
;---------------------------------------
    sta     HMOVE
    stx     GRP0
    stx     GRP1
    lda     faceLLst+2
    sta     COLUP1
    lda     faceRLst+0
    sta     COLUP0
    lda     faceRLst+0
    SLEEP   6
    ldx     faceLLst+1+3
    stx     COLUBK
    tsx                     ; 2     GREEN
    stx     COLUPF
    sta     COLUBK
    ldx     faceRLst+1+3
    sta     COLUPF
    stx     COLUBK
    lda     faceRLst+2+3
    sta     COLUPF
    ldx     #0
    stx     COLUBK

    ldy     #6
.loop4d:
    sta     WSYNC
;---------------------------------------
    sta     HMOVE
    lda     faceLLst+0+3
    sta     COLUPF
    lda     PxPat,y
    sta     GRP0
    sta     GRP1
    SLEEP   3
    lda     faceRLst+0+3
    ldx     faceLLst+2+3
    txs
    ldx     faceLLst+1+3
    stx     COLUBK
    tsx                     ; 2     CYAN
    stx     COLUPF
    sta     COLUBK
    ldx     faceRLst+1+3
    sta     COLUPF
    stx     COLUBK
    lda     faceRLst+2+3
    sta     COLUPF
    ldx     #0
    stx     COLUBK
    dey
    bpl     .loop4d

    lda     faceLLst+0+3
    sta     COLUPF
    SLEEP   2
;---------------------------------------
    sta     HMOVE           ; 3     @3
    stx     GRP0
    stx     GRP1
    lda     faceLLst+0+3
    sta     COLUP1
    lda     faceRLst+2+3
    sta     COLUP0
    lda     faceRLst+0+3
    ldx     faceLLst+1+3
    ldy     #$60
    sty     HMP1
    stx.w   COLUBK
    tsx                     ; 2     CYAN
    stx     COLUPF
    sta     COLUBK
    ldx     faceRLst+1+3
    sta     COLUPF
    stx     COLUBK
    lda     faceRLst+2+3
    sta     COLUPF
    ldx     #0
    stx     COLUBK

    ldy     #$a0
    sty     HMP0

;***********************************************************
; the same code again, just with different colors:
    ldy     #6
.loop5a:
    sta     WSYNC
    sta     HMOVE           ; 3         move outside
    lda     faceLLst+0+3
    sta     COLUPF
    SLEEP   18
    lda     faceRLst+0+3
    ldx     faceLLst+1+3
    stx     COLUBK
    tsx                     ; 2     CYAN
    stx     COLUPF
    sta     COLUBK
    ldx     faceRLst+1+3
    sta     COLUPF
    stx     COLUBK
    lda     faceRLst+2+3
    sta     COLUPF
    ldx     #0
    stx     COLUBK
    dey
    bpl     .loop5a

    sta     WSYNC
    sta     HMOVE
    lda     faceLLst+0+3
    sta     COLUPF
    lda     faceRLst+0+3
    SLEEP   8
    ldx     faceLLst+1+3
    ldy     #$e0
    sty     HMP1
    ldy     #$20
    sty     HMP0
    stx     COLUBK
    tsx                     ; 2     CYAN
    stx     COLUPF
    sta     COLUBK
    ldx     faceRLst+1+3
    sta     COLUPF
    stx     COLUBK
    lda     faceRLst+2+3
    sta     COLUPF
    ldx     #0
    stx     COLUBK

    ldy     #6
.loop5b:
    sta     WSYNC
    sta     HMOVE
    lda     faceLLst+0+6
    sta     COLUPF
    lda     PxPat,y
    sta     GRP0
    sta     GRP1
    SLEEP   8
    lda     faceRLst+0+3
    ldx     faceLLst+1+3
    stx     COLUBK
    tsx                     ; 2     CYAN
    stx     COLUPF
    sta     COLUBK
    ldx     faceRLst+1+3
    sta     COLUPF
    stx     COLUBK
    lda     faceRLst+2+6
    sta     COLUPF
    ldx     #0
    stx     COLUBK
    dey
    bpl     .loop5b

    lda     faceLLst+0+6
    sta     COLUPF
    SLEEP   2
    sta     HMOVE
    stx     GRP0
    stx     GRP1
    lda     faceLLst+1+3
    sta     COLUP1
    lda     faceRLst+1+3
    sta     COLUP0
    lda     faceRLst+0+3
    SLEEP   6
    ldx     faceLLst+1+3
    stx     COLUBK
    tsx                     ; 2     CYAN
    stx     COLUPF
    sta     COLUBK
    ldx     faceRLst+1+3
    sta     COLUPF
    stx     COLUBK
    lda     faceRLst+2+6
    sta     COLUPF
    ldx     #0
    stx     COLUBK

    ldy     #6
.loop5c:
    sta     WSYNC
    sta     HMOVE
    lda     faceLLst+0+6
    sta     COLUPF
    lda     PxPat,y
    sta     GRP0
    sta     GRP1
    SLEEP   8
    lda     faceRLst+0+3
    ldx     faceLLst+1+6
    stx     COLUBK
    tsx                     ; 2     CYAN
    stx     COLUPF
    sta     COLUBK
    ldx     faceRLst+1+6
    sta     COLUPF
    stx     COLUBK
    lda     faceRLst+2+6
    sta     COLUPF
    ldx     #0
    stx     COLUBK
    dey
    bpl     .loop5c

    lda     faceLLst+0+6
    sta     COLUPF
    SLEEP   2
    sta     HMOVE
    stx     GRP0
    stx     GRP1
    lda     faceLLst+2+3
    sta     COLUP1
    lda     faceRLst+0+3
    sta     COLUP0
    lda     faceRLst+0+3
    SLEEP   6
    ldx     faceLLst+1+6
    stx     COLUBK
    tsx                     ; 2     CYAN
    stx     COLUPF
    sta     COLUBK
    ldx     faceRLst+1+6
    sta     COLUPF
    stx     COLUBK
    lda     faceRLst+2+6
    sta     COLUPF
    ldx     #0
    stx     COLUBK

    ldy     #6
.loop5d:
    sta     WSYNC
    sta     HMOVE
    lda     faceLLst+0+6
    sta     COLUPF
    lda     PxPat,y
    sta     GRP0
    sta     GRP1
    SLEEP   3
    lda     faceRLst+0+6
    ldx     faceLLst+2+6
    txs
    ldx     faceLLst+1+6
    stx     COLUBK
    tsx
    stx     COLUPF
    sta     COLUBK
    ldx     faceRLst+1+6
    sta     COLUPF
    stx     COLUBK
    lda     faceRLst+2+6
    sta     COLUPF
    ldx     #0
    stx     COLUBK
    dey
    bpl     .loop5d

    lda     faceLLst+0+6
    sta     COLUPF
    SLEEP   2
    sta     HMOVE           ; 3     @3
    stx     GRP0
    stx     GRP1
    lda     faceLLst+0+6
    sta     COLUP1
    lda     faceRLst+2+6
    sta     COLUP0
    lda     faceRLst+0+6
    ldx     faceLLst+1+6
    ldy     #$60
    sty     HMP1
    stx.w   COLUBK
    tsx
    stx     COLUPF
    sta     COLUBK
    ldx     faceRLst+1+6
    sta     COLUPF
    stx     COLUBK
    lda     faceRLst+2+6
    sta     COLUPF
    ldx     #0
    stx     COLUBK

    ldy     #$a0
    sty     HMP0


;***********************************************************
; the same code again, just with different colors:
    ldy     #6
.loop6a:
    sta     WSYNC
    sta     HMOVE           ; 3         move outside
    lda     faceLLst+0+6
    sta     COLUPF
    SLEEP   18
    lda     faceRLst+0+6
    ldx     faceLLst+1+6
    stx     COLUBK
    tsx
    stx     COLUPF
    sta     COLUBK
    ldx     faceRLst+1+6
    sta     COLUPF
    stx     COLUBK
    lda     faceRLst+2+6
    sta     COLUPF
    ldx     #0
    stx     COLUBK
    dey
    bpl     .loop6a

    sta     WSYNC
    sta     HMOVE
    lda     faceLLst+0+6
    sta     COLUPF
    lda     faceRLst+0+6
    SLEEP   8
    ldx     faceLLst+1+6
    ldy     #$e0
    sty     HMP1
    ldy     #$20
    sty     HMP0
    stx     COLUBK
    tsx
    stx     COLUPF
    sta     COLUBK
    ldx     faceRLst+1+6
    sta     COLUPF
    stx     COLUBK
    lda     faceRLst+2+6
    sta     COLUPF
    ldx     #0
    stx     COLUBK

    ldy     #6
.loop6b:
    sta     WSYNC
    sta     HMOVE
    lda     #0
    sta.w   COLUPF
    lda     PxPat,y
    sta     GRP0
    sta     GRP1
    SLEEP   8
    lda     faceRLst+0+6
    ldx     faceLLst+1+6
    stx     COLUBK
    tsx
    stx     COLUPF
    sta     COLUBK
    ldx     faceRLst+1+6
    sta     COLUPF
    stx     COLUBK
    lda     #0
    sta.w   COLUPF
    ldx     #0
    stx     COLUBK
    dey
    bpl     .loop6b

    lda     #0
    sta.w   COLUPF
    SLEEP   2
    sta     HMOVE
    stx     GRP0
    stx     GRP1
    lda     faceLLst+1+6
    sta     COLUP1
    lda     faceRLst+1+6
    sta     COLUP0
    lda     faceRLst+0+6
    SLEEP   6
    ldx     faceLLst+1+6
    stx     COLUBK
    tsx
    stx     COLUPF
    sta     COLUBK
    ldx     faceRLst+1+6
    sta     COLUPF
    stx     COLUBK
    lda     #0
    sta.w   COLUPF
    ldx     #0
    stx     COLUBK

    ldy     #6
.loop6c:
    sta     WSYNC
    sta     HMOVE
    lda     #0
    sta.w   COLUPF
    lda     PxPat,y
    sta     GRP0
    sta     GRP1
    SLEEP   8
    lda     faceRLst+0+6
    ldx     #0
    stx.w   COLUBK
    tsx
    stx     COLUPF
    sta     COLUBK
    ldx     #0
    sta.w   COLUPF
    stx     COLUBK
    lda     #0
    sta.w   COLUPF
    ldx     #0
    stx     COLUBK
    dey
    bpl     .loop6c

    lda     #0
    sta.w   COLUPF
    SLEEP   2
    sta     HMOVE
    stx     GRP0
    stx     GRP1
    lda     faceLLst+2+6
    sta     COLUP1
    lda     faceRLst+0+6
    sta     COLUP0
    lda     faceRLst+0+6
    SLEEP   5
    ldx     #0
    SLEEP   2
    stx     COLUBK
    tsx
    stx     COLUPF
    sta     COLUBK
    ldx     #0
    sta.w   COLUPF
    stx     COLUBK
    lda     #0
    sta.w   COLUPF
    ldx     #0
    stx     COLUBK

    ldy     #6
.loop6d:
    sta     WSYNC
    sta     HMOVE
    lda     PxPat,y
    sta     GRP0
    sta     GRP1
    dey
    bpl     .loop6d


.exitKernel:
    sta     WSYNC
    lda     #0
    sta     PF1
    sta     PF2
    sta     GRP0
    sta     GRP1

;-------------------------------------

    ldx     #2
.waitScreen:
    lda     INTIM
    bne     .waitScreen
    sta     WSYNC
    stx     VBLANK

    ldx     tmpVar          ; restore stack!!!
    txs
    rts


OverScan SUBROUTINE
    lda     #36
    sta     TIM64T

    ldx     #4
.loopPos:
    lda     PosTbl,x
    jsr     SetPosX
    dex
    bpl     .loopPos
    sta     WSYNC
    sta     HMOVE

    lda     #%1000
    sta     REFP1
    lsr
    sta     REFP0

    lda     #%100001        ; reflected PF and quad sized ball
    sta     CTRLPF

    lda     #%110110        ; 2 pixel missiles/two copies medium
    sta     NUSIZ0
    sta     NUSIZ1

    sta     HMCLR

    lda     #$20            ; move 2 pixels left
    sta     HMP0
    lda     #$e0            ; move 2 pixels right
    sta     HMP1

.waitTim:
    lda     INTIM
    bne     .waitTim
    rts


    align 256

SetPosX SUBROUTINE
; calculates the values and positions objects:
; Input:
; - a = x-position
    tay                      ; 2
    iny                      ; 2
    tya                      ; 2
    and    #$0F              ; 2
    sta    tmpVar2           ; 3
    tya                      ; 2
    lsr                      ; 2
    lsr                      ; 2
    lsr                      ; 2
    lsr                      ; 2
    tay                      ; 2
    clc                      ; 2
    adc    tmpVar2           ; 3
    cmp    #$0F              ; 2
    bcc    .skipIny          ; 2
    sbc    #$0F              ; 2
    iny                      ; 2
.skipIny:
    sta    WSYNC             ; 3
    eor    #$07              ; 2
    asl                      ; 2
    asl                      ; 2
    asl                      ; 2
    asl                      ; 2
    sta.wx  HMP0,x           ; 5        @15
.waitPos:
    dey                      ; 2
    bpl    .waitPos          ; 2
.bankCheck:
    sta    RESP0,X           ; 4
    rts                      ; 6
;SetPosX END


;===============================================================================
; R O M - T A B L E S (Bank 0)
;===============================================================================

    org     $ff00

ColorTbl:
    .byte WHITE, RED, ORANGE, YELLOW, GREEN, BLUE
    .byte CYAN, MAGENTA

PosTbl
    .byte 76+4,     76-4-64
    .byte 76+4+1-8, 76-4+1-64+8
    .byte 76+1  ; the ball is needed for masking color errors at the vertical center

PxPat:
  IF SMOOTH
    .byte   %10000000
    .byte   %11000000
    .byte   %11100000
    .byte   %11110000
    .byte   %11111000
    .byte   %11111100
    .byte   %11111110
    .byte   %11111111
  ELSE
    .byte   %00000000
    .byte   %11000000
    .byte   %01100000
    .byte   %11110000
    .byte   %01111000
    .byte   %11111100
    .byte   %01111110
    .byte   %11111111
  ENDIF

PxPat2:
    .byte   %11111111
    .byte   %11111100
    .byte   %11110000
    .byte   %11000000

PxPat3:
    .byte   %11111111
    .byte   %00111111
    .byte   %00001111
    .byte   %00000011

    org $fffc
    .word   Start
    .word   Start
